# Generated by Django 2.1.4 on 2019-05-09 14:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("scheduler", "0005_auto_20181219_0934")]

    operations = [
        migrations.AddField(
            model_name="schedule",
            name="last_run",
            field=models.DateTimeField(null=True),
        )
    ]
